import { Injectable, Component, Output, EventEmitter, ViewChild, NgModule } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

var terraformerWktParser = require('terraformer-wkt-parser');
var GeocoderService = /** @class */ (function () {
    function GeocoderService(http) {
        this.http = http;
        this.geocoderBaseUrl = 'https://geodata.nationaalgeoregister.nl/locatieserver/v3';
    }
    GeocoderService.prototype.suggest = function (query, options) {
        var _this = this;
        var fq = '*';
        var start = '0';
        var rows = '10';
        if (options) {
            fq = options.fq || fq;
            start = options.start || start;
            rows = options.rows || rows;
        }
        return this.http.get(this.geocoderBaseUrl + "/suggest?", {
            params: {
                q: query,
                fq: fq,
                start: start,
                rows: rows
            }
        }).toPromise().then(function (suggestResultObject) {
            var collations = _this.parseCollations(suggestResultObject.spellcheck.collations);
            var places = _this.formatPlaces(suggestResultObject);
            return { collations: collations, places: places };
        });
    };
    GeocoderService.prototype.lookup = function (id) {
        var _this = this;
        return this.http.get(this.geocoderBaseUrl + "/lookup?", {
            params: {
                id: id,
                fl: '*',
            }
        }).toPromise().then(function (lookupResultObject) {
            return _this.formatLookupResponse(lookupResultObject);
        });
    };
    GeocoderService.prototype.free = function (searchTerm, options) {
        var _this = this;
        var fq = '*';
        var fl = '*';
        var start = '0';
        var rows = '10';
        if (options) {
            fq = options.fq || fq;
            fl = options.fl || fl;
            start = options.start || start;
            rows = options.rows || rows;
        }
        return this.http.get(this.geocoderBaseUrl + "/free?", {
            params: {
                q: searchTerm,
                fl: fl,
                fq: fq,
                rows: rows
            },
        }).toPromise().then(function (freeResultObject) {
            return _this.formatLookupResponse(freeResultObject);
        });
    };
    GeocoderService.prototype.parseCollations = function (collations) {
        var parsedCollations = [];
        for (var i = 0; i < collations.length; i += 2) {
            var collation = {
                id: i,
                naam: collations[i + 1].misspellingsAndCorrections[1],
                weergavenaam: collations[i + 1].hits + " resultaten gevonden voor <strong> " + collations[i + 1].misspellingsAndCorrections[1] + " </strong>",
                hits: collations[i + 1].hits,
            };
            parsedCollations.push(collation);
        }
        return parsedCollations;
    };
    GeocoderService.prototype.formatPlaces = function (suggestResultObject) {
        var places = suggestResultObject.response.docs.map(function (place) {
            return {
                id: place.id,
                type: place.type,
                weergavenaam: place.weergavenaam,
                score: place.score,
                highlight: suggestResultObject.highlighting[place.id].suggest[0]
            };
        });
        return places;
    };
    GeocoderService.prototype.formatLookupResponse = function (lookupResultObject) {
        var formatted = lookupResultObject.response.docs.map(function (lookupResult) {
            var formattedLookupResult = lookupResult;
            formattedLookupResult.centroide_ll = terraformerWktParser.parse(lookupResult.centroide_ll);
            formattedLookupResult.centroide_rd = terraformerWktParser.parse(lookupResult.centroide_rd);
            formattedLookupResult.geometrie_rd = terraformerWktParser.parse(lookupResult.geometrie_rd);
            formattedLookupResult.geometrie_ll = terraformerWktParser.parse(lookupResult.geometrie_ll);
            return formattedLookupResult;
        });
        return formatted[0] || {};
    };
    return GeocoderService;
}());
GeocoderService.decorators = [
    { type: Injectable },
];
GeocoderService.ctorParameters = function () { return [
    { type: HttpClient, },
]; };
var L = require('leaflet');
var GeocoderComponent = /** @class */ (function () {
    function GeocoderComponent(geocoderService) {
        this.geocoderService = geocoderService;
        this.placeFound = new EventEmitter();
        this.searchInput = '';
        this.places = [];
        this.collations = [];
        this.searchThreshold = 2;
        this.foundPlace = null;
        this.selectedItem = [];
        this.selectedIndex = -1;
    }
    GeocoderComponent.prototype.ngOnInit = function () {
    };
    GeocoderComponent.prototype.ngAfterViewInit = function () {
        L.DomEvent.disableClickPropagation(this.geocoderRef.nativeElement);
    };
    GeocoderComponent.prototype.lookup = function (id) {
        var _this = this;
        this.geocoderService.lookup(id).then(function (lookupObj) {
            _this.foundPlace = lookupObj;
            _this.fillInput(_this.foundPlace.weergavenaam);
            _this.placeFound.emit(_this.foundPlace);
            _this.clearPlaces();
        });
    };
    GeocoderComponent.prototype.suggest = function (event) {
        var _this = this;
        if (event && event.code && event.code === 'Enter') {
            return;
        }
        if (this.searchInput.length > this.searchThreshold) {
            this.geocoderService.suggest(this.searchInput).then(function (suggestResponse) {
                _this.places = suggestResponse.places;
                _this.collations = suggestResponse.collations;
            });
        }
        else {
            this.clearPlaces();
        }
    };
    GeocoderComponent.prototype.clear = function () {
        this.foundPlace = null;
        this.searchInput = '';
        this.clearPlaces();
    };
    GeocoderComponent.prototype.clearPlaces = function () {
        this.resetIndex();
        this.places = [];
        this.collations = [];
    };
    GeocoderComponent.prototype.handleEnter = function () {
        if (this.selectedIndex === -1) {
            return;
        }
        var selectedPlace = this.places[this.selectedIndex];
        this.lookup(selectedPlace.id);
    };
    GeocoderComponent.prototype.isNoResultsFound = function () {
        var reachedThreshold = this.searchInput.length > this.searchThreshold;
        var noSuggestions = this.places.length === 0;
        var noResult = (this.foundPlace == null);
        return (reachedThreshold) && (noSuggestions) && (noResult);
    };
    GeocoderComponent.prototype.showCollations = function () {
        return this.collations.length > 0 && this.places.length === 0;
    };
    GeocoderComponent.prototype.fillInput = function (content) {
        this.searchInput = content;
    };
    GeocoderComponent.prototype.isHighlighted = function (i) {
        if (i === this.selectedIndex) {
            return true;
        }
    };
    GeocoderComponent.prototype.moveUp = function () {
        if (this.selectedIndex > 0) {
            this.selectedIndex--;
        }
    };
    GeocoderComponent.prototype.moveDown = function () {
        if (this.selectedIndex < this.places.length) {
            this.selectedIndex++;
        }
    };
    GeocoderComponent.prototype.resetIndex = function () {
        this.selectedIndex = -1;
    };
    GeocoderComponent.prototype.canQuery = function () {
        if (this.searchInput.length > this.searchThreshold) {
            return true;
        }
    };
    GeocoderComponent.prototype.canClear = function () {
        if (this.searchInput.length > 0) {
            return true;
        }
    };
    return GeocoderComponent;
}());
GeocoderComponent.decorators = [
    { type: Component, args: [{
                styles: ["::-webkit-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:-moz-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}::-moz-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:-ms-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}::-ms-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:placeholder-shown{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:focus{outline:0}input,li,p,span{margin:0;font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}.geocoder-container{width:100%;position:relative}.geocoder-search-container{width:100%;border:none;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);background:#fff}.geocoder-search-input{padding:15px;border:none;width:262px;overflow:hidden}button{position:absolute;cursor:pointer}.searchButton{top:13px;right:50px;padding-right:15px;background:0 0;border:none;border-right:1px solid #797979;height:20px}.closeButton{top:10px;right:10px;background:0 0;border:none;height:26px}svg{width:18px;color:#797979}.closeButton:hover svg,.highlight svg,.searchButton:hover svg{color:#79abff;-webkit-transition:all .1s cubic-bezier(.075,.82,.165,1);transition:all .1s cubic-bezier(.075,.82,.165,1)}.geocoder-suggestions-results{top:53px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%}.geocoder-collations-list,.geocoder-suggestions-list{list-style-type:none;padding:0;margin:0}.geocoder-suggestion{padding:15px;position:relative;border-bottom:1px solid #f1f1f1}.geocoder-suggestion-title{max-width:243px}.geocoder-suggestion-type{text-transform:uppercase;font-weight:700;font-size:11px;position:absolute;right:15px;top:36%;color:#bfbfbf}.geocoder-collation:hover,.geocoder-suggestion:hover{cursor:pointer;background-color:#f7f7f7}.focus-background{background-color:#f7f7f7}.geocoder-no-results{top:53px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%;width:calc(100% - 30px);padding:15px}.geocoder-collation-container{top:108px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%}.geocoder-collation-container li{padding:15px;border-bottom:1px solid #f1f1f1}"],
                template: "<div  #geocoder\n      (keydown.ArrowUp)=\"moveUp()\"\n      (keydown.ArrowDown)=\"moveDown()\"\n      (keydown.Enter)=\"handleEnter()\"\n      class=\"geocoder-container\"\n      >\n  <div class=\"geocoder-search-container\">\n    <input class=\"geocoder-search-input\"\n          placeholder=\"Zoek op adres, postcode of perceel...\"\n          [(ngModel)]=\"searchInput\"\n          (keyup)=\"suggest($event)\"\n          (focus)=\"suggest($event)\"\n          type=\"text\"\n          maxlength=\"100\">\n    <button class=\"searchButton\" (click)=\"suggest($event)\" [ngClass]=\"{'highlight': canQuery()}\"> <svg aria-hidden=\"true\" data-fa-processed=\"\" data-prefix=\"fal\" data-icon=\"search\" role=\"img\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 512 512\" class=\"svg-inline--fa fa-search fa-w-16 fa-9x\"><path fill=\"currentColor\" d=\"M508.5 481.6l-129-129c-2.3-2.3-5.3-3.5-8.5-3.5h-10.3C395 312 416 262.5 416 208 416 93.1 322.9 0 208 0S0 93.1 0 208s93.1 208 208 208c54.5 0 104-21 141.1-55.2V371c0 3.2 1.3 6.2 3.5 8.5l129 129c4.7 4.7 12.3 4.7 17 0l9.9-9.9c4.7-4.7 4.7-12.3 0-17zM208 384c-97.3 0-176-78.7-176-176S110.7 32 208 32s176 78.7 176 176-78.7 176-176 176z\" class=\"\"></path></svg></button>\n    <button class=\"closeButton\" (click)=\"clear()\" [ngClass]=\"{'highlight': canClear()}\"> <svg aria-hidden=\"true\" data-fa-processed=\"\" data-prefix=\"fal\" data-icon=\"times\" role=\"img\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 384 512\" class=\"svg-inline--fa fa-times fa-w-12 fa-7x\"><path fill=\"currentColor\" d=\"M217.5 256l137.2-137.2c4.7-4.7 4.7-12.3 0-17l-8.5-8.5c-4.7-4.7-12.3-4.7-17 0L192 230.5 54.8 93.4c-4.7-4.7-12.3-4.7-17 0l-8.5 8.5c-4.7 4.7-4.7 12.3 0 17L166.5 256 29.4 393.2c-4.7 4.7-4.7 12.3 0 17l8.5 8.5c4.7 4.7 12.3 4.7 17 0L192 281.5l137.2 137.2c4.7 4.7 12.3 4.7 17 0l8.5-8.5c4.7-4.7 4.7-12.3 0-17L217.5 256z\" class=\"\"></path></svg></button>\n  </div>\n  <div class=\"geocoder-suggestions-container\">\n    <div class=\"geocoder-suggestions-results\" *ngIf=\"places.length > 0\">\n      <ul class=\"geocoder-suggestions-list\">\n        <li class=\"geocoder-suggestion\"\n            tabindex={{i}}\n            *ngFor=\"let place of places; let i = index;\"\n            (click)=\"lookup(place.id)\"\n            [ngClass]=\"{'focus-background': isHighlighted(i)}\">\n            <p class=\"geocoder-suggestion-title\" [innerHTML]=\"place.highlight\"></p>\n            <p class=\"geocoder-suggestion-type\"> {{place.type}} </p>\n        </li>\n      </ul>\n    </div>\n    <div class=\"geocoder-no-results\" *ngIf=\"isNoResultsFound()\">\n      <p>Geen zoekresultaten gevonden voor <strong> {{this.searchInput}}</strong>.</p>\n    </div>\n    <div class=\"geocoder-collation-container\" *ngIf=\"showCollations()\">\n      <ul class=\"geocoder-collations-list\">\n        <li class=\"geocoder-collation\"\n            tabindex={{i}}\n            (click)=\"fillInput(collation.naam);suggest()\"\n            *ngFor=\"let collation of collations; let i = index;\"\n            [innerHTML]=\"collation.weergavenaam\">\n        </li>\n      </ul>\n    </div>\n  </div>\n</div>\n",
                selector: 'geocoder'
            },] },
];
GeocoderComponent.ctorParameters = function () { return [
    { type: GeocoderService, },
]; };
GeocoderComponent.propDecorators = {
    "geocoderRef": [{ type: ViewChild, args: ['geocoder',] },],
    "placeFound": [{ type: Output },],
};
var GeocoderModule = /** @class */ (function () {
    function GeocoderModule() {
    }
    return GeocoderModule;
}());
GeocoderModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule, FormsModule, CommonModule],
                exports: [GeocoderComponent],
                declarations: [GeocoderComponent],
                providers: [GeocoderService]
            },] },
];

export { GeocoderModule, GeocoderService, GeocoderComponent as ɵa };
//# sourceMappingURL=angular-geocoder.js.map
