import { Component, Input, Output, EventEmitter, OnInit,  ViewChild, ElementRef, OnDestroy, HostListener, OnChanges, SimpleChanges} from '@angular/core';
import { GeocoderService } from './geocoder.service';
import { FormControl, FormGroup } from '@angular/forms';
import { debounceTime, switchMap, distinctUntilChanged, filter } from 'rxjs/operators';
import { Subscription } from 'rxjs';
import { SuggestOptions } from './locatieserver.model';
// Leaflet requires the old school way for importing

@Component({
  styleUrls: ['geocoder.component.scss'],
  templateUrl: 'geocoder.component.html',
  selector: 'geocoder'
})

export class GeocoderComponent implements OnInit, OnDestroy, OnChanges {
  @ViewChild('geocoder') private geocoderRef: ElementRef;
  @Input() type: string;
  @Input() searchInput: string;
  @Input() placeholder: string;
  @Input() resultCountLimit = 10;
  @Input() resultFieldsToShow: Array<string> = [];
  @Output() placeFound: EventEmitter<any> = new EventEmitter<any>();
  public formattedPlaceholder = '';
  public resultsVisible = true;
  public searchField: FormControl;
  public searchForm: FormGroup;

  public places = [];
  public collations = [];
  public searchThreshold = 2;
  public foundPlace: any = null;
  public selectedItem = [];
  public selectedIndex = -1;

  private subscriptions: Subscription[] = [];

  @HostListener('document:click', ['$event']) clickout(event) {
    if (this.geocoderRef.nativeElement.contains(event.target)) {
      this.showResults();
    } else {
      this.hideResults();
    }
  }

  constructor(public geocoderService: GeocoderService) {
    this.searchField = new FormControl();
    this.searchForm = new FormGroup({search: this.searchField});

    this.subscriptions.push(
      this.subscribeToSearchFieldValueChanges(),
    );
  }

  ngOnInit() {
    this.formattedPlaceholder = this.formatPlaceholder(this.placeholder);
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes.searchInput) {
      this.searchField.setValue(changes.searchInput.currentValue);
    }
  }

  ngOnDestroy() {
    this.subscriptions.forEach(s => s.unsubscribe());
  }

  private subscribeToSearchFieldValueChanges() {
    return this.searchField.valueChanges.pipe(
      filter(searchInput => searchInput.length > this.searchThreshold),
      debounceTime(100),
      distinctUntilChanged(),
      switchMap(searchInput => this.suggest(searchInput))
    ).subscribe(suggestResponse => {
      this.places = suggestResponse.places;
      this.collations = suggestResponse.collations;
    }); // Actual call is fired.
  }

  private suggest(searchTerm: string) {
    const flArray = ['id', 'score', 'type', 'weergavenaam', ...this.resultFieldsToShow];
    const params = {
      fq: '*:*',
      fl: flArray.toString(),
      rows: this.resultCountLimit
    } as SuggestOptions;
    if (this.type) {
      const replace = ',';
      const regex = new RegExp(replace, 'g');
      const query = this.type.replace(regex, ' OR ');
      params.fq = `type:(${query})`;
    }
    return this.geocoderService.suggest$(searchTerm, params);
  }

  public free() {
    const searchTerm = this.searchField.value;
    const params = {fq: '*:*'};
    if (this.type) {
      const replace = ',';
      const regex = new RegExp(replace, 'g');
      const query = this.type.replace(regex, ' OR ');
      params.fq = `type:(${query})`;
    }
    return this.geocoderService.free$(searchTerm, params).subscribe(places => {
      this.places = places;
    });
  }

  public lookup(id: string) {
    this.geocoderService.lookup$(id).subscribe((lookupObj) => {
      this.foundPlace = lookupObj;

      if (this.resultFieldsToShow.length > 0) {
        const input = this.resultFieldsToShow.map((resultFieldToShow) => {
          return this.foundPlace[resultFieldToShow];
        }).join(', ');
        this.fillInput(input);
      } else {
        this.fillInput(this.foundPlace.weergavenaam);
      }
      this.placeFound.emit(this.foundPlace);
      this.clearPlaces();
    });
  }

  public clear() {
    this.foundPlace = null;
    this.searchField.setValue('');
    this.clearPlaces();
  }

  public clearPlaces() {
    this.resetIndex();
    this.places = [];
    this.collations = [];
  }


  public handleEnter() {
    if (this.selectedIndex === -1) {
      return;
    }
    const selectedPlace = this.places[this.selectedIndex];
    this.lookup(selectedPlace.id);
  }

  public isNoResultsFound() {
    const reachedThreshold = (this.searchField.value && this.searchField.value.length) > this.searchThreshold;
    const noSuggestions = this.places.length === 0;
    const noResult = (this.foundPlace == null);
    return (reachedThreshold) && (noSuggestions) && (noResult);
  }

  public showCollations() {
    return this.collations.length > 0 && this.places.length === 0;
  }

  public fillInput(content: string, emitEvent = false) {
    this.searchField.setValue(content, {
      emitEvent: emitEvent
    });
  }

  public isHighlighted(i) {
    if (i === this.selectedIndex) {
      return true;
    }
  }

  public moveUp() {
    if (this.selectedIndex > 0) {
      this.selectedIndex--;
    }
  }

  public moveDown() {
    if (this.selectedIndex < this.places.length) {
      this.selectedIndex++;
    }
  }

  public resetIndex() {
    this.selectedIndex = -1;
  }

  public canQuery() {
    const searchInput = this.searchField.value;
    return searchInput && searchInput.length > this.searchThreshold;
  }

  public canClear() {
    const searchInput = this.searchField.value;
    return searchInput && searchInput.length > 0;
  }

  public formatPlaceholder(placeholder: string) {
    let placeholderText = 'Zoeken op de kaart...';

    if (placeholder) {
      placeholderText = placeholder;
    }
    return placeholderText;
  }

  public onCollationClick(name: string) {
    this.fillInput(name, true);
  }

  public showResults() {
    this.resultsVisible = true;
  }

  private hideResults() {
    this.resultsVisible = false;
  }
}
